@@ -1,3 +1,4 @@
+#include <pybind11/functional.h>
 #include <pybind11/numpy.h>
 #include <pybind11/pybind11.h>
 #include <pybind11/stl.h>
@@ -12,6 +13,7 @@
 #include <dune/common/fvector.hh>
 
 #include <array>
+#include <functional>
 #include <memory>
 #include <string>
 
@@ -19,6 +21,13 @@ namespace py = pybind11;
 
 namespace parafields {
 
+template<typename T>
+struct CustomRNG
+{
+  std::function<T()> rng;
+  T sample() { return rng(); }
+};
+
 #if !MPI_IS_FAKEMPI
 #define ADD_MPI_CONSTRUCTOR(dim, t)                                            \
   field##dim##d_##t.def(py::init([](Dune::ParameterTree tree,                  \
@@ -47,11 +56,36 @@ namespace parafields {
                                                                                \
   ADD_MPI_CONSTRUCTOR(dim, t);                                                 \
                                                                                \
+  field##dim##d_##t.def(                                                       \
+    "compute_covariance",                                                      \
+    [](RandomField##dim##D_##t& self,                                          \
+       std::function<t(const t, const std::array<t, dim>&)> func) {            \
+      self.fillMatrix(func);                                                   \
+    });                                                                        \
+                                                                               \
+  field##dim##d_##t.def(                                                       \
+    "add_trend_component",                                                     \
+    [](RandomField##dim##D_##t& self, Dune::ParameterTree tree) {              \
+      self.add_trend_components(tree);                                         \
+    });                                                                        \
+                                                                               \
+  field##dim##d_##t.def(                                                       \
+    "remove_trend_component",                                                  \
+    [](RandomField##dim##D_##t& self) { self.remove_trend_components(1); });   \
+                                                                               \
   field##dim##d_##t.def("generate",                                            \
                         [](RandomField##dim##D_##t& self, unsigned int seed) { \
                           self.generate(seed, true);                           \
                         });                                                    \
                                                                                \
+  field##dim##d_##t.def("generate_with_rng",                                   \
+                        [](RandomField##dim##D_##t& self,                      \
+                           unsigned int seed,                                  \
+                           std::function<t()> rng) {                           \
+                          CustomRNG<t> wrapped{ rng };                         \
+                          self.generateWithRNG(wrapped, true);                 \
+                        });                                                    \
+                                                                               \
   field##dim##d_##t.def(                                                       \
     "probe",                                                                   \
     [](const RandomField##dim##D_##t& self, const py::array_t<t>& pos) {       \
